// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: phonenumber.proto
// Protobuf C++ Version: 7.36.0

#ifndef phonenumber_2eproto_2epb_2eh
#define phonenumber_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_util.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_phonenumber_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_phonenumber_2eproto {
  static const ::uint32_t offsets[];
};
namespace i18n {
namespace phonenumbers {
enum PhoneNumber_CountryCodeSource : int;
extern const uint32_t PhoneNumber_CountryCodeSource_internal_data_[];
class PhoneNumber;
struct PhoneNumberGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern PhoneNumberGlobalsTypeInternal PhoneNumber_globals_;
extern const ::google::protobuf::internal::ClassDataLite PhoneNumber_class_data_;
#else
extern const PhoneNumberGlobalsTypeInternal PhoneNumber_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace phonenumbers
}  // namespace i18n
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::i18n::phonenumbers::PhoneNumber_CountryCodeSource_internal_data_>
    internal::EnumTraitsImpl::value<::i18n::phonenumbers::PhoneNumber_CountryCodeSource>;
}  // namespace protobuf
}  // namespace google

namespace i18n {
namespace phonenumbers {
enum PhoneNumber_CountryCodeSource : int {
  PhoneNumber_CountryCodeSource_UNSPECIFIED = 0,
  PhoneNumber_CountryCodeSource_FROM_NUMBER_WITH_PLUS_SIGN = 1,
  PhoneNumber_CountryCodeSource_FROM_NUMBER_WITH_IDD = 5,
  PhoneNumber_CountryCodeSource_FROM_NUMBER_WITHOUT_PLUS_SIGN = 10,
  PhoneNumber_CountryCodeSource_FROM_DEFAULT_COUNTRY = 20,
};

extern const uint32_t PhoneNumber_CountryCodeSource_internal_data_[];
inline constexpr PhoneNumber_CountryCodeSource PhoneNumber_CountryCodeSource_CountryCodeSource_MIN =
    static_cast<PhoneNumber_CountryCodeSource>(0);
inline constexpr PhoneNumber_CountryCodeSource PhoneNumber_CountryCodeSource_CountryCodeSource_MAX =
    static_cast<PhoneNumber_CountryCodeSource>(20);
[[nodiscard]] inline bool PhoneNumber_CountryCodeSource_IsValid(int value) {
  return 0 <= value && value <= 20 && ((1049635u >> value) & 1) != 0;
}
inline constexpr int PhoneNumber_CountryCodeSource_CountryCodeSource_ARRAYSIZE = 20 + 1;
[[nodiscard]] const ::std::string& PhoneNumber_CountryCodeSource_Name(PhoneNumber_CountryCodeSource value);
template <typename T>
[[nodiscard]] const ::std::string& PhoneNumber_CountryCodeSource_Name(T value) {
  static_assert(::std::is_same<T, PhoneNumber_CountryCodeSource>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to CountryCodeSource_Name().");
  return PhoneNumber_CountryCodeSource_Name(static_cast<PhoneNumber_CountryCodeSource>(value));
}
[[nodiscard]] bool PhoneNumber_CountryCodeSource_Parse(
    ::absl::string_view name, PhoneNumber_CountryCodeSource* PROTOBUF_NONNULL value);
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED PhoneNumber final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:i18n.phonenumbers.PhoneNumber) */ {
  using Super_ = ::google::protobuf::MessageLite;

 public:
  inline PhoneNumber() : PhoneNumber(nullptr) {}
  ~PhoneNumber() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PhoneNumber* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PhoneNumber));
  }
#endif

  template <typename = void>
  explicit constexpr PhoneNumber(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline PhoneNumber(const PhoneNumber& from) : PhoneNumber(nullptr, from) {}
  inline PhoneNumber(PhoneNumber&& from) noexcept : PhoneNumber(nullptr, ::std::move(from)) {}
  inline PhoneNumber& operator=(const PhoneNumber& from) {
    CopyFrom(from);
    return *this;
  }
  inline PhoneNumber& operator=(PhoneNumber&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const PhoneNumber& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<PhoneNumber>(&PhoneNumber_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(PhoneNumber& a, PhoneNumber& b) { a.Swap(&b); }
  inline void Swap(PhoneNumber* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PhoneNumber* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] PhoneNumber* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<PhoneNumber>(arena);
  }
  void CopyFrom(const PhoneNumber& from);
  void MergeFrom(const PhoneNumber& from) { PhoneNumber::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PhoneNumber* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "i18n.phonenumbers.PhoneNumber"; }

  explicit PhoneNumber(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PhoneNumber(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PhoneNumber& from);
  PhoneNumber(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PhoneNumber&& from) noexcept
      : PhoneNumber(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------
  using CountryCodeSource = PhoneNumber_CountryCodeSource;
  static constexpr CountryCodeSource UNSPECIFIED = PhoneNumber_CountryCodeSource_UNSPECIFIED;
  static constexpr CountryCodeSource FROM_NUMBER_WITH_PLUS_SIGN = PhoneNumber_CountryCodeSource_FROM_NUMBER_WITH_PLUS_SIGN;
  static constexpr CountryCodeSource FROM_NUMBER_WITH_IDD = PhoneNumber_CountryCodeSource_FROM_NUMBER_WITH_IDD;
  static constexpr CountryCodeSource FROM_NUMBER_WITHOUT_PLUS_SIGN = PhoneNumber_CountryCodeSource_FROM_NUMBER_WITHOUT_PLUS_SIGN;
  static constexpr CountryCodeSource FROM_DEFAULT_COUNTRY = PhoneNumber_CountryCodeSource_FROM_DEFAULT_COUNTRY;
  [[nodiscard]] static inline bool CountryCodeSource_IsValid(int value) {
    return PhoneNumber_CountryCodeSource_IsValid(value);
  }
  static constexpr CountryCodeSource CountryCodeSource_MIN = PhoneNumber_CountryCodeSource_CountryCodeSource_MIN;
  static constexpr CountryCodeSource CountryCodeSource_MAX = PhoneNumber_CountryCodeSource_CountryCodeSource_MAX;
  static constexpr int CountryCodeSource_ARRAYSIZE = PhoneNumber_CountryCodeSource_CountryCodeSource_ARRAYSIZE;
  template <typename T>
  [[nodiscard]] static inline const ::std::string& CountryCodeSource_Name(T value) {
    return PhoneNumber_CountryCodeSource_Name(value);
  }
  [[nodiscard]] static inline bool CountryCodeSource_Parse(
      ::absl::string_view name, CountryCodeSource* PROTOBUF_NONNULL value) {
    return PhoneNumber_CountryCodeSource_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kExtensionFieldNumber = 3,
    kRawInputFieldNumber = 5,
    kPreferredDomesticCarrierCodeFieldNumber = 7,
    kNationalNumberFieldNumber = 2,
    kCountryCodeFieldNumber = 1,
    kItalianLeadingZeroFieldNumber = 4,
    kCountryCodeSourceFieldNumber = 6,
    kNumberOfLeadingZerosFieldNumber = 8,
  };
  // optional string extension = 3;
  [[nodiscard]] bool has_extension() const;
  void clear_extension() ;
  [[nodiscard]] const ::std::string& extension() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_extension(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_extension();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_extension();
  void set_allocated_extension(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_extension() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_extension(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_extension();

  public:
  // optional string raw_input = 5;
  [[nodiscard]] bool has_raw_input() const;
  void clear_raw_input() ;
  [[nodiscard]] const ::std::string& raw_input() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_raw_input(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_raw_input();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_raw_input();
  void set_allocated_raw_input(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_raw_input() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_raw_input(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_raw_input();

  public:
  // optional string preferred_domestic_carrier_code = 7;
  [[nodiscard]] bool has_preferred_domestic_carrier_code() const;
  void clear_preferred_domestic_carrier_code() ;
  [[nodiscard]] const ::std::string& preferred_domestic_carrier_code() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_preferred_domestic_carrier_code(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_preferred_domestic_carrier_code();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_preferred_domestic_carrier_code();
  void set_allocated_preferred_domestic_carrier_code(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_preferred_domestic_carrier_code() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_preferred_domestic_carrier_code(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_preferred_domestic_carrier_code();

  public:
  // required uint64 national_number = 2;
  [[nodiscard]] bool has_national_number() const;
  void clear_national_number() ;
  [[nodiscard]] ::uint64_t national_number() const;
  void set_national_number(::uint64_t value);

  private:
  ::uint64_t _internal_national_number() const;
  void _internal_set_national_number(::uint64_t value);

  public:
  // required int32 country_code = 1;
  [[nodiscard]] bool has_country_code() const;
  void clear_country_code() ;
  [[nodiscard]] ::int32_t country_code() const;
  void set_country_code(::int32_t value);

  private:
  ::int32_t _internal_country_code() const;
  void _internal_set_country_code(::int32_t value);

  public:
  // optional bool italian_leading_zero = 4;
  [[nodiscard]] bool has_italian_leading_zero() const;
  void clear_italian_leading_zero() ;
  [[nodiscard]] bool italian_leading_zero() const;
  void set_italian_leading_zero(bool value);

  private:
  bool _internal_italian_leading_zero() const;
  void _internal_set_italian_leading_zero(bool value);

  public:
  // optional .i18n.phonenumbers.PhoneNumber.CountryCodeSource country_code_source = 6;
  [[nodiscard]] bool has_country_code_source() const;
  void clear_country_code_source() ;
  [[nodiscard]] ::i18n::phonenumbers::PhoneNumber_CountryCodeSource country_code_source() const;
  void set_country_code_source(::i18n::phonenumbers::PhoneNumber_CountryCodeSource value);

  private:
  ::i18n::phonenumbers::PhoneNumber_CountryCodeSource _internal_country_code_source() const;
  void _internal_set_country_code_source(::i18n::phonenumbers::PhoneNumber_CountryCodeSource value);

  public:
  // optional int32 number_of_leading_zeros = 8 [default = 1];
  [[nodiscard]] bool has_number_of_leading_zeros() const;
  void clear_number_of_leading_zeros() ;
  [[nodiscard]] ::int32_t number_of_leading_zeros() const;
  void set_number_of_leading_zeros(::int32_t value);

  private:
  ::int32_t _internal_number_of_leading_zeros() const;
  void _internal_set_number_of_leading_zeros(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:i18n.phonenumbers.PhoneNumber)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 8,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PhoneNumber& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr extension_;
    ::google::protobuf::internal::ArenaStringPtr raw_input_;
    ::google::protobuf::internal::ArenaStringPtr preferred_domestic_carrier_code_;
    ::uint64_t national_number_;
    ::int32_t country_code_;
    bool italian_leading_zero_;
    int country_code_source_;
    ::int32_t number_of_leading_zeros_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_phonenumber_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// PhoneNumber

// required int32 country_code = 1;
inline bool PhoneNumber::has_country_code() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void PhoneNumber::clear_country_code() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.country_code_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t PhoneNumber::country_code() const {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.country_code)
  return _internal_country_code();
}
inline void PhoneNumber::set_country_code(::int32_t value) {
  _internal_set_country_code(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.country_code)
}
inline ::int32_t PhoneNumber::_internal_country_code() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.country_code_;
}
inline void PhoneNumber::_internal_set_country_code(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.country_code_ = value;
}

// required uint64 national_number = 2;
inline bool PhoneNumber::has_national_number() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void PhoneNumber::clear_national_number() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.national_number_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t PhoneNumber::national_number() const {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.national_number)
  return _internal_national_number();
}
inline void PhoneNumber::set_national_number(::uint64_t value) {
  _internal_set_national_number(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.national_number)
}
inline ::uint64_t PhoneNumber::_internal_national_number() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.national_number_;
}
inline void PhoneNumber::_internal_set_national_number(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.national_number_ = value;
}

// optional string extension = 3;
inline bool PhoneNumber::has_extension() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void PhoneNumber::clear_extension() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.extension_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& PhoneNumber::extension() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.extension)
  return _internal_extension();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PhoneNumber::set_extension(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.extension_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.extension)
}
inline ::std::string* PROTOBUF_NONNULL PhoneNumber::mutable_extension()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_extension();
  // @@protoc_insertion_point(field_mutable:i18n.phonenumbers.PhoneNumber.extension)
  return _s;
}
inline const ::std::string& PhoneNumber::_internal_extension() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.extension_.Get();
}
inline void PhoneNumber::_internal_set_extension(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.extension_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PhoneNumber::_internal_mutable_extension() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.extension_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PhoneNumber::release_extension() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:i18n.phonenumbers.PhoneNumber.extension)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.extension_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.extension_.Set("", GetArena());
  }
  return released;
}
inline void PhoneNumber::set_allocated_extension(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.extension_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.extension_.IsDefault()) {
    _impl_.extension_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:i18n.phonenumbers.PhoneNumber.extension)
}

// optional bool italian_leading_zero = 4;
inline bool PhoneNumber::has_italian_leading_zero() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void PhoneNumber::clear_italian_leading_zero() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.italian_leading_zero_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline bool PhoneNumber::italian_leading_zero() const {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.italian_leading_zero)
  return _internal_italian_leading_zero();
}
inline void PhoneNumber::set_italian_leading_zero(bool value) {
  _internal_set_italian_leading_zero(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.italian_leading_zero)
}
inline bool PhoneNumber::_internal_italian_leading_zero() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.italian_leading_zero_;
}
inline void PhoneNumber::_internal_set_italian_leading_zero(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.italian_leading_zero_ = value;
}

// optional int32 number_of_leading_zeros = 8 [default = 1];
inline bool PhoneNumber::has_number_of_leading_zeros() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void PhoneNumber::clear_number_of_leading_zeros() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.number_of_leading_zeros_ = 1;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int32_t PhoneNumber::number_of_leading_zeros() const {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.number_of_leading_zeros)
  return _internal_number_of_leading_zeros();
}
inline void PhoneNumber::set_number_of_leading_zeros(::int32_t value) {
  _internal_set_number_of_leading_zeros(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.number_of_leading_zeros)
}
inline ::int32_t PhoneNumber::_internal_number_of_leading_zeros() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.number_of_leading_zeros_;
}
inline void PhoneNumber::_internal_set_number_of_leading_zeros(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.number_of_leading_zeros_ = value;
}

// optional string raw_input = 5;
inline bool PhoneNumber::has_raw_input() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void PhoneNumber::clear_raw_input() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.raw_input_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& PhoneNumber::raw_input() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.raw_input)
  return _internal_raw_input();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PhoneNumber::set_raw_input(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.raw_input_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.raw_input)
}
inline ::std::string* PROTOBUF_NONNULL PhoneNumber::mutable_raw_input()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_raw_input();
  // @@protoc_insertion_point(field_mutable:i18n.phonenumbers.PhoneNumber.raw_input)
  return _s;
}
inline const ::std::string& PhoneNumber::_internal_raw_input() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.raw_input_.Get();
}
inline void PhoneNumber::_internal_set_raw_input(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.raw_input_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PhoneNumber::_internal_mutable_raw_input() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.raw_input_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PhoneNumber::release_raw_input() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:i18n.phonenumbers.PhoneNumber.raw_input)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.raw_input_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.raw_input_.Set("", GetArena());
  }
  return released;
}
inline void PhoneNumber::set_allocated_raw_input(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.raw_input_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.raw_input_.IsDefault()) {
    _impl_.raw_input_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:i18n.phonenumbers.PhoneNumber.raw_input)
}

// optional .i18n.phonenumbers.PhoneNumber.CountryCodeSource country_code_source = 6;
inline bool PhoneNumber::has_country_code_source() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  return value;
}
inline void PhoneNumber::clear_country_code_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.country_code_source_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::i18n::phonenumbers::PhoneNumber_CountryCodeSource PhoneNumber::country_code_source() const {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.country_code_source)
  return _internal_country_code_source();
}
inline void PhoneNumber::set_country_code_source(::i18n::phonenumbers::PhoneNumber_CountryCodeSource value) {
  _internal_set_country_code_source(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.country_code_source)
}
inline ::i18n::phonenumbers::PhoneNumber_CountryCodeSource PhoneNumber::_internal_country_code_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::i18n::phonenumbers::PhoneNumber_CountryCodeSource>(_impl_.country_code_source_);
}
inline void PhoneNumber::_internal_set_country_code_source(::i18n::phonenumbers::PhoneNumber_CountryCodeSource value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::i18n::phonenumbers::PhoneNumber_CountryCodeSource_internal_data_));
                                          _impl_.country_code_source_ = value;
}

// optional string preferred_domestic_carrier_code = 7;
inline bool PhoneNumber::has_preferred_domestic_carrier_code() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void PhoneNumber::clear_preferred_domestic_carrier_code() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.preferred_domestic_carrier_code_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& PhoneNumber::preferred_domestic_carrier_code() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:i18n.phonenumbers.PhoneNumber.preferred_domestic_carrier_code)
  return _internal_preferred_domestic_carrier_code();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PhoneNumber::set_preferred_domestic_carrier_code(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.preferred_domestic_carrier_code_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:i18n.phonenumbers.PhoneNumber.preferred_domestic_carrier_code)
}
inline ::std::string* PROTOBUF_NONNULL PhoneNumber::mutable_preferred_domestic_carrier_code()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_preferred_domestic_carrier_code();
  // @@protoc_insertion_point(field_mutable:i18n.phonenumbers.PhoneNumber.preferred_domestic_carrier_code)
  return _s;
}
inline const ::std::string& PhoneNumber::_internal_preferred_domestic_carrier_code() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.preferred_domestic_carrier_code_.Get();
}
inline void PhoneNumber::_internal_set_preferred_domestic_carrier_code(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.preferred_domestic_carrier_code_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PhoneNumber::_internal_mutable_preferred_domestic_carrier_code() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.preferred_domestic_carrier_code_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PhoneNumber::release_preferred_domestic_carrier_code() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:i18n.phonenumbers.PhoneNumber.preferred_domestic_carrier_code)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.preferred_domestic_carrier_code_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.preferred_domestic_carrier_code_.Set("", GetArena());
  }
  return released;
}
inline void PhoneNumber::set_allocated_preferred_domestic_carrier_code(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.preferred_domestic_carrier_code_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.preferred_domestic_carrier_code_.IsDefault()) {
    _impl_.preferred_domestic_carrier_code_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:i18n.phonenumbers.PhoneNumber.preferred_domestic_carrier_code)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace phonenumbers
}  // namespace i18n


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::i18n::phonenumbers::PhoneNumber_CountryCodeSource> : std::true_type {};
template <>
struct internal::LiteEnumFuncs<::i18n::phonenumbers::PhoneNumber_CountryCodeSource> {
  static constexpr bool kIsDefined = true;
  static constexpr auto kParseFunc = ::i18n::phonenumbers::PhoneNumber_CountryCodeSource_Parse;
  static constexpr auto kNameFunc = ::i18n::phonenumbers::PhoneNumber_CountryCodeSource_Name<int>;
};

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // phonenumber_2eproto_2epb_2eh
